#
#   _===_ _           _                  ______ _     _             _ _ _
#  |  ___| |         | |                 | ___ (_)   | |           (_) | |
#  | |__ | | ___  ___| |_ _ __ ___ ______| |_/ /_  __| | ___  _   _ _| | | ___ _   _ _ __
#  |  __|| |/ _ \/ __| __| '__/ _ \______| ___ \ |/ _` |/ _ \| | | | | | |/ _ \ | | | '__|
#  | |___| |  __/ (__| |_| | | (_) |     | |_/ / | (_| | (_) | |_| | | | |  __/ |_| | |
#  \____/|_|\___|\___|\__|_|  \___/      \____/|_|\__,_|\___/ \__,_|_|_|_|\___|\__,_|_|
#
#  Essais de la mini-plaquette CP2112 via USB d'ordinateur
#  Code Source Python 3 utilisant les librairies CP2112 de Silicon-Labs incluses dans le
#  SDK USBXpress Host (version linux, Windows ou MaC) disponible au: https://www.silabs.com/interface/usb-bridges/classic/device.cp2112 .
#  Électro-Bidouilleur, Décembre 2021. http://bidouilleur.ca
#  Licence de droits: Attribution 4.0 International (CC BY 4.0)  Lisez https://creativecommons.org/licenses/by/4.0/deed.fr

#import SLABHIDDevice   # Pas requis, l'interface HID est géréé à même la librairie SLABHIDtoSMBUS
import SLABHIDtoSMBUS
import time

vid = 0x10c4    # Le CP2112 Vendor ID
pid = 0xea90    # Le CP2112 Product ID

i2c_master_address = 0x03   # L'adresse i2c du maître CP2112
i2c_slave_address = 0x48    # L'adresse i2c de l'esclâve ADS1113

try:
    h = SLABHIDtoSMBUS.HidSmbusDevice()     # Initialisation de la classe HidSmbusDevice
    h.Open(0,vid, pid)           # Connexion au CP2112
    print("No. de Série: " + h.GetString())
    print("Pour I2C Ouvert: " + str(h.IsOpened()))

    # Configuration du CP2112 - Bus I2C (SMBus)    SetSmbusConfig(bitRate, master_address, autoReadRespond, writeTimeout, readTimeout, sclLowTimeout, transferRetries)
    h.SetSmbusConfig(100000, 2, False, 0, 0, False, 0)

    # Configuration du CP2112 - Entrées/sorties GPIO.   SetGpioConfig( direction, mode, function, clkDiv):
    h.SetGpioConfig(0b11111111, 0b11111111, 0b00000110, 0)

    # Configuration de l'ADC ADS1113     WriteRequest(slave_address, data_to_send_as_array, byte_count)
    h.WriteRequest(i2c_slave_address<<1, [1,0b01000100,0b10000011], 3)

    while(True):
        # Lancer une demande de lecture de l'ADC     AddressReadRequest(slave_address, byte_count, slave_reg_address_size, slave_reg_address)
        h.AddressReadRequest(i2c_slave_address<<1, 2, 1, 0)
        # Bogue du CP2112 version F01 !!! Il faut faire un "Data Read Force Send" pour forcer la donnée lue dans le registre "Data Read Response".
        h.ForceReadResponse(2)
        # Ensuite lire la lecture d'ADC   GetReadResponse(slave_address, byte_count):
        returned_data = h.GetReadResponse(i2c_slave_address<<1,64)
        # Extraire la valeur 16 bits d'échantillon ADC des données reçues
        adc_val = (returned_data[0]<<8) + returned_data[1]
        # Afficher la lecture ADC à l'écran
        print("ADC Reading: {} \n".format(adc_val))

        # Gérer les valeurs des sorties GPIO
        # D'abord lire la valeur courante des GPIO                                          
        gpio = h.ReadLatch()
        # Afficher la valeur inversée                                           
        print("GPIO Values: {0:#010b}".format(gpio^0xFF))
        # Inverser et écrire la nouvelle valeur GPIO au port                                           
        h.WriteLatch(gpio^0xFF, 0xFF)  #        
        # Pause pour pouvoir lire à l'écran
        time.sleep(0.5)
# Gérer les erreurs        
except IOError as ex:
    print(ex)
    print("Error. Exiting....")

